##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, random, string, calendar
from Globals import InitializeClass, DTMLFile, package_home
from Persistence import Persistent
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
try:
    from Products.Localizer import Gettext
    use_localizer = 1
except:
    use_localizer = 0

_zptdir = os.path.join(package_home(globals()), 'www')

if use_localizer:
    _ = Gettext.translation(globals())
    N_ = Gettext.dummy

    #Date Strings to be translated
    N_('Monday')
    N_('Tuesday')
    N_('Wednesday')
    N_('Thursday')
    N_('Friday')
    N_('Saturday')
    N_('Sunday')

    #Short date strings to be translated
    N_('Mo')
    N_('Tu')
    N_('We')
    N_('Th')
    N_('Fr')
    N_('Sa')
    N_('Su')

    #Month Strings to be translated
    N_('January')
    N_('February')
    N_('March')
    N_('April')
    N_('May')
    N_('June')
    N_('July')
    N_('August')
    N_('September')
    N_('October')
    N_('November')
    N_('December')

class MediaManagerException( Exception ):
    pass

class TabItem(Persistent):
    """
    An object contaning the information for creating a tab object.
    The tab can either link to a page on the portal or to another box object.
    """
    boxtool = None
    id = None
    title = None
    type = None
    link = None
    boxid = None
    language = None
    security = ClassSecurityInfo()

    def __init__(self, boxtool, id, title='', type='', link='', boxid=''):
        self.boxtool = boxtool
        self.id = id
        self.title = title
        self.type = type
        self.link = link
        self.boxid = boxid
        self.language = None

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id of the tab object.
        """
        return self.id

    security.declarePublic('getTitle')
    def getTitle(self):
        """
        Return the title of the tab object.
        """
        return self.title

    security.declarePublic('setTitle')
    def setTitle(self, title=None):
        """
        Update the title of the tab object.
        """
        if title is not None:
            self.title = title

    security.declarePublic('getType')
    def getType(self):
        """
        Return the type of link that the tab object links to (can be either link or box).
        """
        return self.type

    security.declarePublic('setType')
    def setType(self, type=None):
        """
        Update the type of link that the tab object links to (can be either link or box).
        """
        if type is not None:
            self.type = type

    security.declarePublic('getLink')
    def getLink(self):
        """
        Return the link that the tab object links to.
        """
        return self.link

    security.declarePublic('setLink')
    def setLink(self, link=None):
        """
        Update the link that the tab object links to.
        """
        if link is not None:
            self.link = link

    security.declarePublic('getBoxId')
    def getBoxId(self):
        """
        Return the id of the box that the tab object links to.
        """
        return self.boxid

    security.declarePublic('setBoxId')
    def setBoxId(self, boxid=None):
        """
        Update the id of the box that the tab object links to.
        """
        if boxid is not None:
            self.boxid = boxid

    security.declarePublic('getLanguage')
    def getLanguage(self):
        """
        Return the language settings for this tab object.
        If set to 'None', the tab object will turn up for all languages.
        """
        return self.language

    security.declarePublic('setLanguage')
    def setLanguage(self, language=None):
        """
        Update the language settings for this tab object.
        If set to 'None', the tab object will turn up for all languages.
        """
        if language is not None:
            if language == 'None':
                self.language = None
            else:
                self.language = language

class CalendarBox(Persistent):

    """
    An object containing the information for creating a calendar box.
    """
    boxtool = None
    id = None
    title = None
    itemCount = None
    subjects = None
    types = None
    sort_criteria = None
    format_types = None
    language = None

    security = ClassSecurityInfo()
    type = 'Calendar Box'
    meta_type = 'Box Tool Calendar Box'

    def __init__(self, boxtool, id, title='', itemCount=0, subjects=[], types=[], sort_criteria='', format_types=[], language=''):
        self.boxtool = boxtool
        self.id = id
        self.title = title
        self.itemCount = itemCount
        self.subjects = subjects
        self.types = types
        self.sort_criteria = sort_criteria
        self.format_types = format_types
        self.language = language

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id of the calendarbox object.
        """
        return self.id

    security.declarePublic('getType')
    def getType(self):
        """
        Return the type of the calendarbox object.
        Default set to 'Calendar Box'.
        """
        return self.type
        
    security.declarePublic('getMetaType')
    def getMetaType(self):
        """
        Return the meta type
        """
        return self.meta_type

    security.declarePublic('getTitle')
    def getTitle(self):
        """
        Return the title of the calendarbox object.
        """
        return self.title

    security.declarePublic('setTitle')
    def setTitle(self, title=None):
        """
        Update the title of the calendarbox object.
        """
        if title is not None:
            self.title = title

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the calendarbox object.
        (How many items will show up in the lists).
        """
        return self.itemCount

    security.declarePublic('setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the calendarbox object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            self.itemCount = itemCount

    security.declarePublic('getSubjects')
    def getSubjects(self):
        """
        Returns the subjects for the calendarbox object.
        """
        return self.subjects

    security.declarePublic('setSubjects')
    def setSubjects(self, subjects=None):
        """
        Update the subjects for the calendarbox object.
        """
        if subjects is not None:
            self.subjects = subjects

    security.declarePublic('getTypes')
    def getTypes(self):
        """
        Return the types selected for the calendarbox object.
        Contains a list of the portal types that should turn up in the calendar box.
        """
        return self.types

    security.declarePublic('setTypes')
    def setTypes(self, types=None):
        """
        Update the portal types that will turn up in the calendar box.
        """
        if types is not None:
            self.types = types

    security.declarePublic('getSortCriteria')
    def getSortCriteria(self):
        """
        Returns the sort criteria used for this calendarbox object.
        """
        return self.sort_criteria

    security.declarePublic('setSortCriteria')
    def setSortCriteria(self, sort_criteria=None):
        """
        Update the sort criteria used for this calendarbox object.
        """
        if sort_criteria is not None:
            self.sort_criteria = sort_criteria

    security.declarePublic('getFormatTypes')
    def getFormatTypes(self):
        """
        Return the format settings used when displaying the content of the calendarbox object.
        """
        return self.format_types

    security.declarePublic('setFormatTypes')
    def setFormatTypes(self, format_types=None):
        """
        Update the format settings used when displaying the content of the calendarbox object.
        """
        if format_types is not None:
            self.format_types = format_types

    security.declarePublic('getLanguage')
    def getLanguage(self):
        """
        Return the language used when searching for content for the calendarbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """
        return self.language

    security.declarePublic('setLanguage')
    def setLanguage(self, language=None):
        """
        Update the language used when searching for content for the calendarbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """
        if language is not None:
            if language == 'None':
                self.language = None
            else:
                self.language = language

    security.declarePublic('getUseLanguage')
    def getUseLanguage(self):
        """
        Return a boolean indicating whether the language settings should be used
        when searching for the content of the calendarbox object.
        """
        if hasattr(self, 'use_language'):
            return self.use_language
        else:
            return 0

    security.declarePublic('setUseLanguage')
    def setUseLanguage(self, use_language):
        """
        Update the setting indicating whether the language settings should be used
        when searching for the content of the calendarbox object.
        The use_language attribute is a boolean.
        """
        self.use_language = int(use_language)

    security.declarePublic('getPrevDate')
    def getPrevDate(self, date=None, mode='month'):
        """
        Returns the previous date used for creating links to previous month/day
        depending on the mode used when calling the function.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        year = date.year()
        month = date.month()
        day = date.day()

        if mode == 'day':
            rangem = month - 1
            monthrange = calendar.monthrange(year, rangem)
            lastday = monthrange[1]
            if month == 1 and day == 1:
                nyear = year - 1
                nmonth = 12
                nday = 31
            elif day == 1:
                nyear = year
                nmonth = month - 1
                nday = lastday
            else:
                nyear = year
                nmonth = month
                nday = day - 1
            ndate = DateTime(nyear, nmonth, nday)
        elif mode == 'month':
            if month == 1:
                nyear = year - 1
                nmonth = 12
            else:
                nyear = year
                nmonth = month - 1
            ndate = DateTime(nyear, nmonth, 1)
        return ndate

    security.declarePublic('getNextDate')
    def getNextDate(self, date=None, mode='month'):
        """
        Returns the next date used for creating links to next month/day
        depending on the mode used when calling the function.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        year = date.year()
        month = date.month()
        day = date.day()

        if mode == 'day':
            monthrange = calendar.monthrange(year, month)
            lastday = monthrange[1]
            if month == 12 and day == 31:
                nyear = year + 1
                nmonth = 1
                nday = 1
            elif day == monthrange[1]:
                nyear = year
                nmonth = month + 1
                nday = 1
            else:
                nyear = year
                nmonth = month
                nday = day + 1
            ndate = DateTime(nyear, nmonth, nday)
        elif mode == 'month':
            if month == 12:
                nyear = year + 1
                nmonth = 1
            else:
                nyear = year
                nmonth = month + 1
            ndate = DateTime(nyear, nmonth, 1)
        return ndate

    security.declarePublic('getHeadline')
    def getHeadline(self, date=None, mode='month'):
        """
        Return the headline for the calendarbox object.
        If you have Localizer installed, this function will translate the day- and monthnames.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                zLOG.LOG('Box Tool - get headline', zLOG.INFO, 'Date is: %s. Type is: %s' % (str(date), type(date)))
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        #zLOG.LOG('Box Tool - Get headline', zLOG.INFO, 'Date is: %s\nMode is: %s' % (str(date), mode))

        if mode == 'day':
            #zLOG.LOG('Box Tool - get headline', zLOG.INFO, 'Mode is day')
            dname = date.Day()
            if use_localizer:
                dname = _(dname)
            daydate = date.dd()
            mname = date.Month()
            if use_localizer:
                mname = _(mname)
            yname = date.year()
            mname = str(dname) + ' ' + str(daydate) + ' ' + str(mname) + ' ' + str(yname)
        elif mode == 'month':
            #zLOG.LOG('Box Tool - get headline', zLOG.INFO, 'Mode is month')
            mname = date.Month()
            if use_localizer:
                mname = _(mname)
            year = date.year()
            mname = str(mname) + ' ' + str(year)
        else:
            zLOG.LOG('Box Tool - get headline', zLOG.INFO, 'Mode is: %s' % mode)
        return mname

    security.declarePublic('getDateDict')
    def getDateDict(self, date=None, mode='month'):
        """
        Return a dictionary with the dates for displaying either a month view or a day view.
        The dictionary is build with 2 entries: 'daynames' and 'daylist'.
        The 'daynames' entry contains a list of the weekdays (if month view) or the name of the weekday (if day view).
        The 'daylist' entry contains a list of weeks which contains a list of day dictionaries.
        These day dictionaries contains 2 entries: 'daynr' and 'date'.
        The 'daynr' entry contains the number of the day for displaying in the calendar.
        The 'date' entry contains the actual date for the daynr entry.
        The lists will be created for both month view and date view.
        If you have Localizer installed, the day names will be translated.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        list = {}

        if mode == 'day':
            dayname = date.Day()
            if use_localizer:
                dayname = _(dayname)
            daynames = []
            list['daynames'] = daynames.append(dayname)
            montharr = []
            week = []
            day = {}
            daynr = date.day()
            date = date
            day['daynr'] = daynr
            day['date'] = date
            week.append(day)
            montharr.append(week)
            list['daylist'] = montharr
        elif mode == 'month':
            calendar.setfirstweekday(calendar.MONDAY)
            if use_localizer:
                daynames = []
                daynames.append(_('Mo'))
                daynames.append(_('Tu'))
                daynames.append(_('We'))
                daynames.append(_('Th'))
                daynames.append(_('Fr'))
                daynames.append(_('Sa'))
                daynames.append(_('Su'))
            else:
                daynames = ['Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su']
            list['daynames'] = daynames
            year = int(date.year())
            month = int(date.mm())
            monthlist = calendar.monthcalendar(year, month)
            montharr = []
            for weekentry in monthlist:
                week = []
                for dayentry in weekentry:
                    day = {}
                    if dayentry == 0:
                        day['daynr'] = ''
                        day['date'] = None
                        week.append(day)
                    else:
                        day['daynr'] = dayentry
                        day['date'] = DateTime(year, month, dayentry)
                        week.append(day)
                montharr.append(week)
            list['daylist'] = montharr
        return list

class BoxItem(Persistent):
    """
    An object containing the information for creating a list box.
    """
    id = None
    title = None
    itemCount = None
    subjects = None
    types = None
    sort_criteria = None
    format_types = None
    language = None

    security = ClassSecurityInfo()
    type = 'List Box'

    def __init__(self, id, title='', itemCount=0, subjects=[], types=[], sort_criteria='', format_types=[], language=''):
        self.id = id
        self.title = title
        self.itemCount = itemCount
        self.subjects = subjects
        self.types = types
        self.sort_criteria = sort_criteria
        self.format_types = format_types
        self.language = language

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id of the listbox object.
        """
        return self.id

    security.declarePublic('getType')
    def getType(self):
        """
        Return the type of the listbox object.
        Default set to 'List Box'.
        """
        return self.type

    security.declarePublic('getTitle')
    def getTitle(self):
        """
        Return the title for the listbox object.
        """
        return self.title

    security.declarePublic('setTitle')
    def setTitle(self, title=None):
        """
        Update the title for the listbox object.
        """
        if title is not None:
            self.title = title

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the listbox object.
        (How many items will show up in the box).
        """
        return self.itemCount

    security.declarePublic('setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the listbox object.
        Defines how many items will show up in the box.
        """
        if itemCount is not None:
            self.itemCount = itemCount

    security.declarePublic('getSubjects')
    def getSubjects(self):
        """
        Returns the subjects selected for the listbox object.
        """
        return self.subjects

    security.declarePublic('setSubjects')
    def setSubjects(self, subjects=None):
        """
        Update the subjects for the listbox object.
        """
        if subjects is not None:
            self.subjects = subjects

    security.declarePublic('getTypes')
    def getTypes(self):
        """
        Return the list of selected portal types to be used when searching for content to the listbox object.
        """
        return self.types

    security.declarePublic('setTypes')
    def setTypes(self, types=None):
        """
        Update the list of portal types to be used when searching for content to the listbox object.
        """
        if types is not None:
            self.types = types

    security.declarePublic('getSortCriteria')
    def getSortCriteria(self):
        """
        Return the sort criteria used when searching for the content to the listbox object.
        """
        return self.sort_criteria

    security.declarePublic('setSortCriteria')
    def setSortCriteria(self, sort_criteria=None):
        """
        Set the sort criteria used when searching for the content to the listbox object.
        """
        if sort_criteria is not None:
            self.sort_criteria = sort_criteria

    security.declarePublic('getFormatTypes')
    def getFormatTypes(self):
        """
        Return the format settings for formatting an entry in the listbox object.
        """
        return self.format_types

    security.declarePublic('setFormatTypes')
    def setFormatTypes(self, format_types=None):
        """
        Set the format settings for an entry in the listbox object.
        """
        if format_types is not None:
            self.format_types = format_types

    security.declarePublic('getLanguage')
    def getLanguage(self):
        """
        Return the language used when searching for content for the listbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """
        return self.language

    security.declarePublic('setLanguage')
    def setLanguage(self, language=None):
        """
        Update the language used when searching for content for the listbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """
        if language is not None:
            if language == 'None':
                self.language = None
            else:
                self.language = language

    security.declarePublic('getUseLanguage')
    def getUseLanguage(self):
        """
        Return a boolean indicating whether the language settings should be used
        when searching for the content of the listbox object.
        """
        if hasattr(self, 'use_language'):
            return self.use_language
        else:
            return 0

    security.declarePublic('setUseLanguage')
    def setUseLanguage(self, use_language):
        """
        Update the setting indicating whether the language settings should be used
        when searching for the content of the listbox object.
        The use_language attribute is a boolean.
        """
        self.use_language = int(use_language)

class BoxTool( UniqueObject, SimpleItem ):

    """
    A tool for displaying boxes :-)
    """

    id = 'portal_boxtool'
    meta_type = 'Box Tool'

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,
                     ) + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainBoxTool', _zptdir)

    #
    #   'portal_boxtool' interface methods
    #

    boxformat_types = [{'name':'Icon', 'type':'icon'}
                     , {'name':'Title', 'type':'title'}
                     , {'name':'Description', 'type':'description'}
                     , {'name':'Date', 'type':'date'}
                     , {'name':'Author', 'type': 'author'}]

    midleft_slotlist = []
    midright_slotlist = []
    right_slotlist = []

    security.declarePublic('listBoxItems')
    def listBoxItems(self):
        """
        Returns all the box objects created in the tool.
        """
        if hasattr(self, 'box_items'):
            return self.box_items
        else:
            return None

    security.declarePublic('listSelectedTabItems')
    def listSelectedTabItems(self):
        """
        Returns all the tab objects in the tool for the selected language.
        """
        try:
            msgcatalog = self.mmm_messages
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        tabs = []
        if hasattr(self, 'tab_items'):
            for tab in self.tab_items:
                #zLOG.LOG('Box Tool - list tabs', zLOG.INFO, 'Tab is %s' % str(tab))
                lan = tab.getLanguage()
                if lan == sel_language or not lan:
                    tabs.append(tab)
            return tabs
        else:
            return None

    security.declarePublic('listTabItems')
    def listTabItems(self):
        """
        Returns all the tab objects created in the tool.
        """
        if hasattr(self, 'tab_items'):
            return self.tab_items
        else:
            return None

    security.declarePublic('getSlotBoxes')
    def getSlotBoxes(self, slot):
        """
        Return the box objects in the selected language for the requested slot.
        """
        try:
            msgcatalog = self.mmm_messages
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        #zLOG.LOG('Box Tool - Get slot boxes', zLOG.INFO, 'Lang variable contains: %s for slot %s' % (sel_language, slot))
        boxes = []
        slotlist = None
        if slot == 'mid-left':
            slotlist = self.midleft_slotlist
        elif slot == 'mid-right':
            slotlist = self.midright_slotlist
        elif slot == 'right':
            slotlist = self.right_slotlist

        if slotlist is None:
            raise 'Attribute Error', 'Slot %s was not found' % slot

        for boxid in slotlist:
            box = self.getBoxItem(boxid)
            #zLOG.LOG('Box Tool', zLOG.INFO, 'Box %s has language: %s' % (str(box.getTitle()), str(box.getLanguage())))
            if box.getLanguage() == sel_language or not box.getLanguage():
                boxes.append(boxid)
                #zLOG.LOG('Box Tool', zLOG.INFO, 'Box %s added to list' % str(box.getTitle()))
        #zLOG.LOG('Box Tool - Get slot boxes', zLOG.INFO, 'Returning boxes: %s' % str(boxes))
        return boxes

    security.declarePublic('getDefaultBox')
    def getDefaultBox(self):
        """
        Returns the default box object for use in the index_html template.
        """
        try:
            return self.getSlotBoxes('mid-left')[0]
        except:
            return None

    security.declarePublic('getTabItem')
    def getTabItem(self, id=None):
        """
        Returns the requested tab object.
        """
        item = None
        if id is not None:
            for tabitem in self.tab_items:
                if tabitem.id == id:
                    item = tabitem

        else:
            raise 'Attribute Error', 'This function needs an id in order to return a tab item'

        return item

    security.declarePublic('getBoxItem')
    def getBoxItem(self, id=None):
        """
        Returns the requested box object.
        """
        item = None
        if id is not None:
            for boxitem in self.box_items:
                #zLOG.LOG('Box Tool - Get Box Item', zLOG.INFO, 'We have an item: %s' % boxitem.id)
                if boxitem.id == id:
                    item = boxitem
                    #zLOG.LOG('Box Tool', zLOG.INFO, 'We found a box item !!')
            if not item:
                if id in self.midleft_slotlist:
                    index = 0
                    for item in self.midleft_slotlist:
                        if item == id:
                            del self.midleft_slotlist[index]
                            self._p_changed = 1
                        index = index + 1
                elif id in self.midright_slotlist:
                    index = 0
                    for item in self.midright_slotlist:
                        if item == id:
                            del self.midright_slotlist[index]
                            self._p_changed = 1
                        index = index + 1
                elif id in self.right_slotlist:
                    index = 0
                    for item in self.right_slotlist:
                        if item == id:
                            del self.right_slotlist[index]
                            self._p_changed = 1
                        index = index + 1
                return None
        else:
            raise 'Attribute Error', 'This function needs an id in order to return a box item'

        return item

    security.declarePublic('addBoxItem')
    def addBoxItem(self, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None):
        """
        Add a new list box object to the tool.
        """
        id = str(DateTime().millis()) + str(random.randint(0, 1000000))
        #zLOG.LOG('Box Tool', zLOG.INFO, 'Add box with new id: %s' % id)
        box_item = BoxItem(id, title, itemCount, subjects, types, sort_criteria, format_types, language)
        if not hasattr(self, 'box_items'):
            self.box_items = []
        self.box_items.append(box_item)
        self._p_changed = 1

    security.declarePublic('addCalendarBox')
    def addCalendarBox(self, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None):
        """
        Add a new calendar box object to the tool.
        """
        id = str(DateTime().millis()) + str(random.randint(0, 1000000))
        box_item = CalendarBox(self, id, title, itemCount, subjects, types, sort_criteria, format_types, language)
        if not hasattr(self, 'box_items'):
            self.box_items = []
        self.box_items.append(box_item)
        self._p_changed = 1

    security.declarePublic('addTabItem')
    def addTabItem(self, title):
        """
        Adds a new tab object to the tool.
        """
        id = str(DateTime().millis()) + str(random.randint(0, 1000000))
        tab_item = TabItem(self, id, title=title)
        if not hasattr(self, 'tab_items'):
            self.tab_items = []
        self.tab_items.append(tab_item)
        self._p_changed = 1

    security.declarePublic('deleteBoxItem')
    def deleteBoxItem(self, index):
        """
        Delete a box object with the requsted index from the tool.
        """
        if not type(index) == type(0):
            index = int(index)

        if index >= 0 and index < len(self.box_items):
            del self.box_items[index]
            self._p_changed = 1

    security.declarePublic('deleteTabItem')
    def deleteTabItem(self, index):
        """
        Delete a tab object with the requested index from the tool.
        """
        if not type(index) == type(0):
            index = int(index)

        if index >=0 and index < len(self.tab_items):
            del self.tab_items[index]
            self._p_changed = 1

    security.declarePublic('editBoxItem')
    def editBoxItem(self, id, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None, use_language=None):
        """
        Edit the requested box object.
        """
        box_item = self.getBoxItem(id)
        if box_item:
            box_item.setTitle(title)
            box_item.setItemCount(itemCount)
            box_item.setSubjects(subjects)
            box_item.setTypes(types)
            box_item.setSortCriteria(sort_criteria)
            box_item.setFormatTypes(format_types)
            box_item.setLanguage(language)
            if use_language is not None:
                box_item.setUseLanguage(use_language)
            self._p_changed = 1

    security.declarePublic('editTabItem')
    def editTabItem(self, id, title=None, type=None, link=None, boxid=None, language=None):
        """
        Edit the requested tab object.
        """
        tab_item = self.getTabItem(id)
        #zLOG.LOG('Box Tool - edit tab', zLOG.INFO, 'Getting tab item %s. Got item %s' % (str(id), str(tab_item)))
        if tab_item:
            tab_item.setTitle(title)
            tab_item.setType(type)
            tab_item.setLink(link)
            tab_item.setBoxId(boxid)
            tab_item.setLanguage(language)

    security.declarePublic('moveupBoxItem')
    def moveupBoxItem(self, index):
        """
        Moved the requested box object one up in the list order.
        """
        if not type(index) == type(0):
            index = int(index)
        if index > 0 and index < len(self.box_items):
            old = self.box_items[index - 1]
            self.box_items[index - 1] = self.box_items[index]
            self.box_items[index] = old
            self._p_changed = 1

    security.declarePublic('moveupTabItem')
    def moveupTabItem(self, index):
        """
        Move the requested tab object one up in the list order.
        """
        if not type(index) == type(0):
            index = int(index)

        if index > 0 and index < len(self.tab_items):
            old = self.tab_items[index - 1]
            self.tab_items[index - 1] = self.tab_items[index]
            self.tab_items[index] = old
            self._p_changed = 1

    security.declarePublic('movedownBoxItem')
    def movedownBoxItem(self, index):
        """
        Moves the requested box object one down in the list order.
        """
        if not type(index) == type(0):
            index = int(index)
        if index < len(self.box_items)-1 and index >= 0:
            old = self.box_items[index + 1]
            self.box_items[index + 1] = self.box_items[index]
            self.box_items[index] = old
            self._p_changed = 1

    security.declarePublic('movedownTabItem')
    def movedownTabItem(self, index):
        """
        Moves down the requested tab object one down in the list order.
        """
        if not type(index) == type(0):
            index = int(index)

        if index < len(self.tab_items)-1 and index >= 0:
            old = self.tab_items[index + 1]
            self.tab_items[index + 1] = self.tab_items[index]
            self.tab_items[index] = old
            self._p_changed = 1

    security.declarePublic('setBoxSlots')
    def setBoxSlots(self, midleftslot=None, midrightslot=None, rightslot=None):
        """
        Sets the box objects that should appear in the different slots.
        """
        if midleftslot is not None:
            self.midleft_slotlist = midleftslot
        if midrightslot is not None:
            self.midright_slotlist = midrightslot
        if rightslot is not None:
            self.right_slotlist = rightslot

    security.declarePublic('isItemInList')
    def isItemInList(self, id):
        """
        Checks whether the requested item is listed in any of the tool's slots.
        """
        if id in self.midleft_slotlist:
            return 1
        else:
            if id in self.midright_slotlist:
                return 1
            else:
                if id in self.right_slotlist:
                    return 1
                else:
                    return 0

    security.declarePublic('getBoxContent')
    def getBoxContent(self, id, date=None):
        """
        Returns a dictionary containg the format and the items
        to be displayed in the box item.
        """
        catalog = getToolByName(self, 'portal_catalog')
        #zLOG.LOG('Box Tool - Get Box Content', zLOG.INFO, 'We want to fetch id: %s' % id)
        box_item = self.getBoxItem(id)
        if not box_item:
            return None
        boxtype = box_item.getType()
        if hasattr(box_item, 'subjects'):
            subj = box_item.subjects
        else:
            subj = None
        if hasattr(box_item, 'types'):
            types = box_item.types
        else:
            types = None
        if hasattr(box_item, 'format_types'):
            boxitemformats = box_item.format_types
            if boxitemformats:
                format_types = []
                for formatset in self.boxformat_types:
                    typo = formatset['type']
                    if typo in boxitemformats:
                        set = {'name': formatset['name'], 'type': typo}
                        format_types.append(set)
            else:
                format_types = self.boxformat_types
        else:
            format_types = self.boxformat_types
        if hasattr(box_item, 'language'):
            language = box_item.language
        use_language = box_item.getUseLanguage()
        #zLOG.LOG('Box Tool', zLOG.INFO, 'Use Language: %s' % str(use_language))
        if hasattr(box_item, 'sort_criteria'):
            sort_criteria = box_item.sort_criteria
        if not sort_criteria:
            sort_criteria = 'Date'

        zLOG.LOG('Box Tool', zLOG.INFO, 'Box item format: %s' % str(format_types))
        result = None
        if types:
            if boxtype == 'List Box':
                if language and use_language:
                    if subj and subj[0]:
                        result = catalog.searchResults(Type=types
                                                     , Subject=subj
                                                     , Language=language
                                                     , sort_on=sort_criteria
                                                     , sort_order='reverse')
                        zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nLanguage: %s\nsort_on: %s' % (str(types), str(subj), str(language), str(sort_criteria)))
                    else:
                        result = catalog.searchResults(Type=types
                                                     , sort_on=sort_criteria
                                                     , Language=language
                                                     , sort_order='reverse')
                        zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nLanguage: %s\nsort_on: %s' % (str(types), str(language), str(sort_criteria)))
                else:
                    if subj and subj[0]:
                        result = catalog.searchResults(Type=types
                                                     , Subject=subj
                                                     , sort_on=sort_criteria
                                                     , sort_order='reverse')
                        zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nsort_on: %s' % (str(types), str(subj), str(sort_criteria)))
                    else:
                        result = catalog.searchResults(Type=types
                                                     , sort_on=sort_criteria
                                                     , sort_order='reverse')
                        zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nsort_on: %s' % (str(types), str(sort_criteria)))
            elif boxtype == 'Calendar Box':
                if date is None:
                    return
                for type in types:
                    if type == 'Live Event' or type == 'Event':
                        if language and use_language:
                            if subj and subj[0]:
                                res = catalog.searchResults(Type=type
                                                          , Subject=subj
                                                          , Language=language
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , start_date=date.latestTime()
                                                          , start_date_usage='range:max'
                                                          , end_date=date.earliestTime()
                                                          , end_date_usage='range:min')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nLanguage: %s\nsort_on: %s\nDate: %s' % (str(type), str(subj), str(language), str(sort_criteria), str(date)))
                            else:
                                res = catalog.searchResults(Type=type
                                                          , sort_on=sort_criteria
                                                          , Language=language
                                                          , sort_order='reverse'
                                                          , start_date=date.latestTime()
                                                          , start_date_usage='range:max'
                                                          , end_date=date.earliestTime()
                                                          , end_date_usage='range:min')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nLanguage: %s\nsort_on: %s\nDate: %s' % (str(type), str(language), str(sort_criteria), str(date)))
                        else:
                            if subj and subj[0]:
                                res = catalog.searchResults(Type=type
                                                          , Subject=subj
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , start_date=date.latestTime()
                                                          , start_date_usage='range:max'
                                                          , end_date=date.earliestTime()
                                                          , end_date_usage='range:min')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nsort_on: %s\nDate: %s' % (str(type), str(subj), str(sort_criteria), str(date)))
                            else:
                                res = catalog.searchResults(Type=type
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , start_date=date.latestTime()
                                                          , start_date_usage='range:max'
                                                          , end_date=date.earliestTime()
                                                          , end_date_usage='range:min')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nsort_on: %s\nDate: %s' % (str(type), str(sort_criteria), str(date)))
                    else:
                        if language and use_language:
                            if subj and subj[0]:
                                res = catalog.searchResults(Type=type
                                                          , Subject=subj
                                                          , Language=language
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , modified=[date.earliestTime(), date.latestTime()]
                                                          , modified_usage='range:min:max')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nLanguage: %s\nsort_on: %s\nDate: %s' % (str(type), str(subj), str(language), str(sort_criteria), str(date)))
                            else:
                                res = catalog.searchResults(Type=type
                                                          , sort_on=sort_criteria
                                                          , Language=language
                                                          , sort_order='reverse'
                                                          , modified=[date.earliestTime(), date.latestTime()]
                                                          , modified_usage='range:min:max')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nLanguage: %s\nsort_on: %s\nDate: %s' % (str(type), str(language), str(sort_criteria), str(date)))
                        else:
                            if subj and subj[0]:
                                res = catalog.searchResults(Type=type
                                                          , Subject=subj
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , modified=[date.earliestTime(), date.latestTime()]
                                                          , modified_usage='range:min:max')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nSubject: %s\nsort_on: %s\nDate: %s' % (str(type), str(subj), str(sort_criteria), str(date)))
                            else:
                                res = catalog.searchResults(Type=type
                                                          , sort_on=sort_criteria
                                                          , sort_order='reverse'
                                                          , modified=[date.earliestTime(), date.latestTime()]
                                                          , modified_usage='range:min:max')
                                zLOG.LOG('Box Tool - get content', zLOG.INFO, 'Search criterias:\nType: %s\nsort_on: %s\nDate: %s' % (str(type), str(sort_criteria), str(date)))
                    if not result:
                        result = res
                    else:
                        if res:
                            result = result + res
            else:
                return None
        zLOG.LOG('Box Tool', zLOG.INFO, 'Results: %s' % str(result))
        items = []
        memtool = getToolByName(self, 'portal_membership')
        if result:
            for entry in result:
                object = entry.getObject()
                if object:
                    item = {}
                    for formatset in format_types:
                        ftype = formatset['type']
                        fname = formatset['name']
                        #zLOG.LOG('Box Tool', zLOG.INFO, 'Format type: %s' % ftype)
                        if ftype == 'icon':
                            try:
                                icon = object.getEventPictureSrc()
                            except:
                                icon = object.getIcon()
                            item[fname] = {'src': icon, 'href': object.absolute_url()}
                        elif ftype == 'title':
                            item[fname] = {'href': object.absolute_url(), 'title': object.title_or_id}
                        elif ftype == 'description':
                            item[fname] = {'text': object.description}
                        elif ftype == 'date':
                            if object.Type() == 'Live Event' or object.Type() == 'Event':
                                startd = object.start_date
                                startd = startd.strftime('%d/%m-%Y %H:%M')
                                endd = object.end_date
                                endd = endd.strftime('%d/%m-%Y %H:%M')
                                text = "%s - %s" % (str(startd), str(endd))
                            else:
                                date = DateTime(object.ModificationDate())
                                text = date.strftime('%d/%m-%Y %H:%M')
                            item[fname] = {'text': text}
                        elif ftype == 'author':
                            homeurl = memtool.getHomeUrl(object.Creator())
                            item[fname] = {'href': homeurl, 'text': object.Creator()}
                    #zLOG.LOG('Box Tool', zLOG.INFO, 'Item contains: %s' % str(item))
                    items.append(item)

            return {'format':format_types, 'entries': items}
        else:
            return None

InitializeClass( BoxTool )
