##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

from Interface import Attribute, Base

class TabItem( Base ):

    """
    An object contaning the information for creating a tab object.
    The tab can either link to a page on the portal or to another box object.
    """

    def getId(self):
        """
        Return the id of the tab object.
        """

    def getTitle(self):
        """
        Return the title of the tab object.
        """

    def setTitle(self, title=None):
        """
        Update the title of the tab object.

        o title: Should be a string. If set to 'None', the title property will not be changed.
        """

    def getType(self):
        """
        Return the type of link that the tab object links to (can be either link or box).
        """

    def setType(self, type=None):
        """
        Update the type of link that the tab object links to.

        o type: Should be a string containing either 'link' or 'box'. If set to 'None', the type property will not be changed.
        """

    def getLink(self):
        """
        Return the link that the tab object links to.
        """

    def setLink(self, link=None):
        """
        Update the link that the tab object links to.

        o link: Should be a string containg a relative path to the page where the link should point to.
        The portal url will be added in front of the path (e.g. http://yourportal.com/).
        If set to 'None', the link property will not be updated.
        """

    def getBoxId(self):
        """
        Return the id of the box that the tab object links to.
        """

    def setBoxId(self, boxid=None):
        """
        Update the id of the box that the tab object links to.

        o boxid: Should be a string containing the id of the box object where the link should point to.
        If set to 'None', the boxid property will not be updated.
        """

    def getLanguage(self):
        """
        Return the language settings for this tab object.
        If set to 'None', the tab object will turn up for all languages.
        """

    def setLanguage(self, language=None):
        """
        Update the language settings for this tab object.

        o language: Should be a string containing the language code for the preferred language (e.g. 'en').
        If set to 'None', the tab object will turn up for all languages.
        """

class CalendarBox( Base ):

    """
    An object containing the information for creating a calendar box.
    """

    def getId(self):
        """
        Return the id of the calendarbox object.
        """

    def getType(self):
        """
        Return the type of the calendarbox object.
        Default set to 'Calendar Box'.
        """

    def getTitle(self):
        """
        Return the title of the calendarbox object.
        """

    def setTitle(self, title=None):
        """
        Update the title of the calendarbox object.
        
        o title: Should be a string. If set to 'None', the title property will not be changed.
        """

    def getItemCount(self):
        """
        Return the item count for the calendarbox object.
        (How many items will show up in the lists).
        """

    def setItemCount(self, itemCount=None):
        """
        Update the item count of the calendarbox object.
        Defines how many items will show up in the day list.

        o itemCount: should be an integer. If set to 'None', the itemCount property will not be changed.
        """

    def getSubjects(self):
        """
        Returns the subjects for the calendarbox object.
        """

    def setSubjects(self, subjects=None):
        """
        Update the subjects for the calendarbox object.
        
        o subjects: Should be a list of subjects present on the portal.
        If set to 'None', the subjects property will not be changed.
        """

    def getTypes(self):
        """
        Return the types selected for the calendarbox object.
        Contains a list of the portal types that should turn up in the calendar box.
        """

    def setTypes(self, types=None):
        """
        Update the portal types that will turn up in the calendar box.
        
        o types. Should be a list of portal types that exists on the portal.
        If set to 'None', the types property will not be changed.
        """

    def getSortCriteria(self):
        """
        Returns the sort criteria used for this calendarbox object.
        """

    def setSortCriteria(self, sort_criteria=None):
        """
        Update the sort criteria used for this calendarbox object.

        o sort_criteria: Should be a string containg a valid sort criteria for the portal.
        If set to 'None', the sort_criteria property will not be changed.
        """

    def getFormatTypes(self):
        """
        Return the format settings used when displaying the content of the calendarbox object.
        """

    def setFormatTypes(self, format_types=None):
        """
        Update the format settings used when displaying the content of the calendarbox object.

        o format_types: Should be a list that contains dictionaries for formatting the content of an entry in the calendar day view.
        If set to 'None', the format_types property will not be updated.
        """

    def getLanguage(self):
        """
        Return the language used when searching for content for the calendarbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """

    def setLanguage(self, language=None):
        """
        Update the language used when searching for content for the calendarbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).

        o language: Should be a string containing the language code for the preferred language (e.g. 'en').
        If set to 'None', the language property will not be used (and the calendarbox object will show up for all languages).
        """

    def getUseLanguage(self):
        """
        Return a boolean indicating whether the language settings should be used
        when searching for the content of the calendarbox object.
        """

    def setUseLanguage(self, use_language):
        """
        Update the setting indicating whether the language settings should be used
        when searching for the content of the calendarbox object.
        
        o use_language: Should be an integer with the value of either 1 (true) or 0 (false).
        """

    def getPrevDate(self, date=None, mode='month'):
        """
        Returns the previous date used for creating links to previous month/day
        depending on the mode used when calling the function.

        o date: Should be the date we calculate from. If not supplied, the current date will be used.

        o mode: Should be a string containing either 'month' or 'day'. If not supplied, the mode will be set to 'month'.
        """

    def getNextDate(self, date=None, mode='month'):
        """
        Returns the next date used for creating links to next month/day
        depending on the mode used when calling the function.

        o date: Should be the date we calculate from. If not supplied, the current date will be used.

        o mode: Should be a string containing either 'month' or 'day'. If not supplied, the mode will be set to 'month'.
        """

    def getHeadline(self, date=None, mode='month'):
        """
        Return the headline for the calendarbox object.
        If you have Localizer installed, this function will translate the day- and monthnames.

        o date: Should be the date we calculate from. If not supplied, the current date will be used.

        o mode: Should be a string containing either 'month' or 'day'. If not supplied, the mode will be set to 'month'.
        """

    def getDateDict(self, date=None, mode='month'):
        """
        Return a dictionary with the dates for displaying either a month view or a day view.
        The dictionary is build with 2 entries: 'daynames' and 'daylist'.
        The 'daynames' entry contains a list of the weekdays (if month view) or the name of the weekday (if day view).
        The 'daylist' entry contains a list of weeks which contains a list of day dictionaries.
        These day dictionaries contains 2 entries: 'daynr' and 'date'.
        The 'daynr' entry contains the number of the day for displaying in the calendar.
        The 'date' entry contains the actual date for the daynr entry.
        The lists will be created for both month view and date view.
        If you have Localizer installed, the day names will be translated.

        o date: Should be the date we calculate from. If not supplied, the current date will be used.

        o mode: Should be a string containing either 'month' or 'day'. If not supplied, the mode will be set to 'month'.
        """

class BoxItem(Base):
    """
    An object containing the information for creating a list box.
    """

    def getId(self):
        """
        Return the id of the listbox object.
        """

    def getType(self):
        """
        Return the type of the listbox object.
        Default set to 'List Box'.
        """

    def getTitle(self):
        """
        Return the title for the listbox object.
        """

    def setTitle(self, title=None):
        """
        Update the title for the listbox object.

        o title: Should be a string. If set to 'None', the title property will not be changed.
        """

    def getItemCount(self):
        """
        Return the item count for the listbox object.
        (How many items will show up in the box).
        """

    def setItemCount(self, itemCount=None):
        """
        Update the item count of the listbox object.
        Defines how many items will show up in the box.

        o itemCount: Should be an integer. If set to 'None', the itemCount ptoperty will no be changed.
        """

    def getSubjects(self):
        """
        Returns the subjects selected for the listbox object.
        """

    def setSubjects(self, subjects=None):
        """
        Update the subjects for the listbox object.
        
        o subjects: Should be a list containing subjects that exist on the portal. If set to 'None', the subjects property will not be changed.
        """

    def getTypes(self):
        """
        Return the list of selected portal types to be used when searching for content to the listbox object.
        """

    def setTypes(self, types=None):
        """
        Update the list of portal types to be used when searching for content to the listbox object.

        o types: Should be a list of portal types that exist on the portal. If set to 'None', the types property will not be changed.
        """

    def getSortCriteria(self):
        """
        Return the sort criteria used when searching for the content to the listbox object.
        """

    def setSortCriteria(self, sort_criteria=None):
        """
        Set the sort criteria used when searching for the content to the listbox object.

        o sort_criteria: Should be a string containing a sort criteria that exist in the 'portal_catalog' tool.
        If set to 'None', the sort_criteria property will not be changed.
        """

    def getFormatTypes(self):
        """
        Return the format settings for formatting an entry in the listbox object.
        """

    def setFormatTypes(self, format_types=None):
        """
        Set the format settings for an entry in the listbox object.

        o format_types: Should be a list that contains dictionaries for formatting the content of an entry in the calendar day view.
        If set to 'None', the format_types property will not be updated.
        """

    def getLanguage(self):
        """
        Return the language used when searching for content for the listbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).
        """

    def setLanguage(self, language=None):
        """
        Update the language used when searching for content for the listbox object.
        Will also be used when displaying boxes (what language setting the box will turn up for).

        o language: Should be a string containing the language code for the preferred language (e.g. 'en').
        If set to 'None', the language property will not be used (and the calendarbox object will show up for all languages).
        """

    def getUseLanguage(self):
        """
        Return a boolean indicating whether the language settings should be used
        when searching for the content of the listbox object.
        """

    def setUseLanguage(self, use_language):
        """
        Update the setting indicating whether the language settings should be used
        when searching for the content of the listbox object.

        o use_language: Should be an integer with the value of either 1 (true) or 0 (false).
        """

class BoxTool( Base ):

    """
    A tool for displaying boxes :-)
    """

    def listBoxItems(self):
        """
        Returns all the box objects created in the tool.
        """

    def listSelectedTabItems(self):
        """
        Returns all the tab objects in the tool for the selected language.
        """

    def listTabItems(self):
        """
        Returns all the tab objects created in the tool.
        """

    def getSlotBoxes(self, slot):
        """
        Return the box objects in the selected language for the requested slot.
        
        o slot: Should be a string containing either 'mid-left', 'mid-right' or 'right'.
        """

    def getDefaultBox(self):
        """
        Returns the default box object for use in the index_html template.
        """

    def getTabItem(self, id=None):
        """
        Returns the requested tab object.
        
        o id: Should be a string containing a valid id of a tab object.
        If set to 'None', an error will be raised.
        """

    def getBoxItem(self, id=None):
        """
        Returns the requested box object.

        o id: Should be a string containing a valid id of a box object.
        If set to 'None', an error will be raised.
        """

    def addBoxItem(self, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None):
        """
        Add a new list box object to the tool.

        o title: Should be a string.

        o itemCount: Should be an integer.

        o subjects: Should be a list.

        o types: Should be a list.

        o sort_criteria: Should be a string.

        o format_types: Should be a list.

        o language: Should be a string.
        """

    def addCalendarBox(self, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None):
        """
        Add a new calendar box object to the tool.

        o title: Should be a string.

        o itemCount: Should be an integer.

        o subjects: Should be a list.

        o types: Should be a list.

        o sort_criteria: Should be a string.

        o format_types: Should be a list.

        o language: Should be a string.
        """

    def addTabItem(self, title):
        """
        Adds a new tab object to the tool.
        
        o title: Should be a string.
        """

    def deleteBoxItem(self, index):
        """
        Delete a box object with the requsted index from the tool.
        
        o index: Should be an integer.
        """

    def deleteTabItem(self, index):
        """
        Delete a tab object with the requested index from the tool.

        o index: Should be an integer.
        """

    def editBoxItem(self, id, title=None, itemCount=None, subjects=None, types=None, sort_criteria=None, format_types=None, language=None, use_language=None):
        """
        Edit the requested box object.

        o id: Should be a string containing a valid id for a box object.
        Updates the box object with the supplied arguments.
        """

    def editTabItem(self, id, title=None, type=None, link=None, boxid=None, language=None):
        """
        Edit the requested tab object.
        
        id: Should be a string containing a valid id for a tab object.
        Updates the tab object with the supplied arguments.
        """

    def moveupBoxItem(self, index):
        """
        Move the requested box object one up in the list order.
        
        o index: Should be an integer.
        """

    def moveupTabItem(self, index):
        """
        Move the requested tab object one up in the list order.
        
        o index: Should be an integer.
        """

    def movedownBoxItem(self, index):
        """
        Moves the requested box object one down in the list order.
        
        o index: Should be an integer.
        """

    def movedownTabItem(self, index):
        """
        Moves down the requested tab object one down in the list order.
        
        o index: Should be an integer.
        """

    def setBoxSlots(self, midleftslot=None, midrightslot=None, rightslot=None):
        """
        Sets the box objects that should appear in the different slots.
        
        o midleftslot: Should be a list containing ids of box objects. If set to 'None', the midleftslot property will not be changed.

        o midrightslot: Should be a list containing ids of box objects. If set to 'None', the midrightslot property will not be changed.

        o rightslot: Should be a list containing ids of box objects. If set to 'None', the rightslot property will not be changed.
        """

    def isItemInList(self, id):
        """
        Checks whether the requested item is listed in any of the tool's slots.
        
        o id: Should be a string containing a valid id of a box object.
        """

    def getBoxContent(self, id, date=None):
        """
        Returns a dictionary containg the format and the items
        to be displayed in the box item.

        o id: Should be a string containing a valid id of a box object.
        
        o date: Should be the date to use when searching for content. If set to 'None', the current date will be used.
        """